@@ -13,6 +13,7 @@ from . import utils as cvu
 from . import defaults as cvd
 from . import base as cvb
 from . import parameters as cvpar
+from . import immunity as cvi
 from collections import defaultdict
 
 
@@ -965,7 +966,7 @@ class contact_tracing(Intervention):
         start_day   (int):        intervention start day (default: 0, i.e. the start of the simulation)
         end_day     (int):        intervention end day (default: no end)
         presumptive (bool):       whether or not to begin isolation and contact tracing on the presumption of a positive diagnosis (default: no)
-        quar_period (int):        number of days to quarantine when notified as a known contact. Default value is pars['quar_period']
+        quar_period (int):        number of days to quarantine when notified as a known contact. Default value is ``pars['quar_period']``
         kwargs      (dict):       passed to Intervention()
 
     **Example**::
@@ -981,7 +982,7 @@ class contact_tracing(Intervention):
         self.start_day   = start_day
         self.end_day     = end_day
         self.presumptive = presumptive
-        self.quar_period = quar_period  #: If quar_period is None, it will be drawn from sim.pars at initialization
+        self.quar_period = quar_period # If quar_period is None, it will be drawn from sim.pars at initialization
         return
 
 
@@ -1094,7 +1095,9 @@ class contact_tracing(Intervention):
             sim: Simulation object
             contacts: {trace_time: np.array(inds)} dictionary storing which people to notify
         '''
+        is_dead = cvu.true(sim.people.dead) # Find people who are not alive
         for trace_time, contact_inds in contacts.items():
+            contact_inds = np.setdiff1d(contact_inds, is_dead) # Do not notify contacts who are dead
             sim.people.known_contact[contact_inds] = True
             sim.people.date_known_contact[contact_inds] = np.fmin(sim.people.date_known_contact[contact_inds], sim.t + trace_time)
             sim.people.schedule_quarantine(contact_inds, start_date=sim.t + trace_time, period=self.quar_period - trace_time)  # Schedule quarantine for the notified people to start on the date they will be notified
@@ -1104,12 +1107,12 @@ class contact_tracing(Intervention):
 
 #%% Treatment and prevention interventions
 
-__all__+= ['vaccine']
+__all__+= ['simple_vaccine', 'vaccinate']
 
 
-class vaccine(Intervention):
+class simple_vaccine(Intervention):
     '''
-    Apply a vaccine to a subset of the population. In addition to changing the
+    Apply a simple vaccine to a subset of the population. In addition to changing the
     relative susceptibility and the probability of developing symptoms if still
     infected, this intervention stores several types of data:
 
@@ -1129,10 +1132,13 @@ class vaccine(Intervention):
         cumulative (bool): whether cumulative doses have cumulative effects (default false); can also be an array for efficacy per dose, with the last entry used for multiple doses; thus True = [1] and False = [1,0]
         kwargs     (dict): passed to Intervention()
 
+    Note: this intervention is still under development and should be used with caution.
+    It is intended for use with use_waning=False.
+
     **Examples**::
 
-        interv = cv.vaccine(days=50, prob=0.3, rel_sus=0.5, rel_symp=0.1)
-        interv = cv.vaccine(days=[10,20,30,40], prob=0.8, rel_sus=0.5, cumulative=[1, 0.3, 0.1, 0]) # A vaccine with efficacy up to the 3rd dose
+        interv = cv.simple_vaccine(days=50, prob=0.3, rel_sus=0.5, rel_symp=0.1)
+        interv = cv.simple_vaccine(days=[10,20,30,40], prob=0.8, rel_sus=0.5, cumulative=[1, 0.3, 0.1, 0]) # A vaccine with efficacy up to the 3rd dose
     '''
     def __init__(self, days, prob=1.0, rel_sus=0.0, rel_symp=0.0, subtarget=None, cumulative=False, **kwargs):
         super().__init__(**kwargs) # Initialize the Intervention object
@@ -1159,6 +1165,7 @@ class vaccine(Intervention):
         self.orig_symp_prob    = sc.dcp(sim.people.symp_prob) # ...and symptom probability
         self.mod_rel_sus       = np.ones(sim.n, dtype=cvd.default_float) # Store the final modifiers
         self.mod_symp_prob     = np.ones(sim.n, dtype=cvd.default_float) # Store the final modifiers
+        self.vacc_inds         = None
         return
 
 
@@ -1169,7 +1176,7 @@ class vaccine(Intervention):
         for ind in find_day(self.days, sim.t, interv=self, sim=sim):
 
             # Construct the testing probabilities piece by piece -- complicated, since need to do it in the right order
-            vacc_probs = np.full(sim.n, self.prob) # Begin by assigning equal testing probability to everyone
+            vacc_probs = np.full(sim.n, self.prob) # Begin by assigning equal vaccination probability to everyone
             if self.subtarget is not None:
                 subtarget_inds, subtarget_vals = get_subtargets(self.subtarget, sim)
                 vacc_probs[subtarget_inds] = subtarget_vals # People being explicitly subtargeted
@@ -1193,4 +1200,168 @@ class vaccine(Intervention):
             for v_ind in vacc_inds:
                 self.vaccination_dates[v_ind].append(sim.t)
 
+            # Update vaccine attributes in sim
+            sim.people.vaccinated[vacc_inds] = True
+            sim.people.vaccinations[vacc_inds] += 1
+
         return
+
+
+class vaccinate(Intervention):
+    '''
+    Apply a vaccine to a subset of the population. In addition to changing the
+    relative susceptibility and the probability of developing symptoms if still
+    infected, this intervention stores several types of data:
+
+        - ``vaccinations``:      the number of vaccine doses per person
+        - ``vaccination_dates``: list of dates per person
+        - ``pars``:             vaccine pars that are given to Vaccine() class
+
+    Args:
+        vaccine (dict/str): which vaccine to use
+        label        (str): if vaccine is supplied as a dict, the name of the vaccine
+        days     (int/arr): the day or array of days to apply the interventions
+        prob       (float): probability of being vaccinated (i.e., fraction of the population)
+        subtarget  (dict): subtarget intervention to people with particular indices (see test_num() for details)
+        kwargs     (dict): passed to Intervention()
+
+    **Examples**::
+
+        pfizer = cv.vaccinate(vaccine='pfizer', days=50, prob=0.3)
+        custom = cv.vaccinate(vaccine=)
+    '''
+    def __init__(self, vaccine, days, label=None, prob=1.0, subtarget=None, **kwargs):
+        super().__init__(**kwargs) # Initialize the Intervention object
+        self.days      = sc.dcp(days)
+        self.prob      = prob
+        self.subtarget = subtarget
+        self.index     = None # Index of the vaccine in the sim; set later
+        self.label     = None # Vacine label (used as a dict key)
+        self.p         = None # Vaccine parameters
+        self.parse(vaccine=vaccine, label=label) # Populate
+        return
+
+
+    def parse(self, vaccine=None, label=None):
+        ''' Unpack vaccine information, which may be given as a string or dict '''
+
+        # Option 1: vaccines can be chosen from a list of pre-defined strains
+        if isinstance(vaccine, str):
+
+            choices, mapping = cvpar.get_vaccine_choices()
+            strain_pars = cvpar.get_vaccine_strain_pars()
+            dose_pars = cvpar.get_vaccine_dose_pars()
+
+            label = vaccine.lower()
+            for txt in ['.', ' ', '&', '-', 'vaccine']:
+                label = label.replace(txt, '')
+
+            if label in mapping:
+                label = mapping[label]
+                vaccine_pars = sc.mergedicts(strain_pars[label], dose_pars[label])
+            else: # pragma: no cover
+                errormsg = f'The selected vaccine "{vaccine}" is not implemented; choices are:\n{sc.pp(choices, doprint=False)}'
+                raise NotImplementedError(errormsg)
+
+            if self.label is None:
+                self.label = label
+
+        # Option 2: strains can be specified as a dict of pars
+        elif isinstance(vaccine, dict):
+
+            # Parse label
+            vaccine_pars = vaccine
+            label = vaccine_pars.pop('label', label) # Allow including the label in the parameters
+            if label is None:
+                label = 'custom'
+
+        else: # pragma: no cover
+            errormsg = f'Could not understand {type(vaccine)}, please specify as a string indexing a predefined vaccine or a dict.'
+            raise ValueError(errormsg)
+
+        # Set label and parameters
+        self.label = label
+        self.p = sc.objdict(vaccine_pars)
+
+        return
+
+
+    def initialize(self, sim):
+        ''' Fix the dates and store the vaccinations '''
+        super().initialize()
+
+        # Populate any missing keys -- must be here, after strains are initialized
+        default_strain_pars = cvpar.get_vaccine_strain_pars(default=True)
+        default_dose_pars   = cvpar.get_vaccine_dose_pars(default=True)
+        strain_labels       = list(sim['strain_pars'].keys())
+        dose_keys           = list(default_dose_pars.keys())
+
+        # Handle dose keys
+        for key in dose_keys:
+            if key not in self.p:
+                self.p[key] = default_dose_pars[key]
+
+        # Handle strains
+        for key in strain_labels:
+            if key not in self.p:
+                if key in default_strain_pars:
+                    val = default_strain_pars[key]
+                else:
+                    val = 1.0
+                    if sim['verbose']: print('Note: No cross-immunity specified for vaccine {self.label} and strain {key}, setting to 1.0')
+                self.p[key] = val
+
+        self.days = process_days(sim, self.days) # days that group becomes eligible
+        self.second_dose_days = [None]*(sim['n_days']+1)  # inds who get second dose (if relevant)
+        self.vaccinated       = [None]*(sim['n_days']+1) # keep track of inds of people vaccinated on each day
+        self.vaccinations      = np.zeros(sim.n, dtype=cvd.default_int) # Number of doses given per person
+        self.vaccination_dates = np.full(sim.n, np.nan) # Store the dates when people are vaccinated
+        sim['vaccine_pars'][self.label] = self.p # Store the parameters
+        self.index = list(sim['vaccine_pars'].keys()).index(self.label) # Find where we are in the list
+        sim['vaccine_map'][self.index]  = self.label # Use that to populate the reverse mapping
+
+        return
+
+
+    def apply(self, sim):
+        ''' Perform vaccination '''
+
+        if sim.t >= np.min(self.days):
+            # Determine who gets first dose of vaccine today
+            vacc_probs = np.zeros(sim.n)
+            if self.subtarget is not None:
+                subtarget_inds, subtarget_vals = get_subtargets(self.subtarget, sim)
+                vacc_probs[subtarget_inds] = subtarget_vals  # People being explicitly subtargeted
+            else:
+                for ind in find_day(self.days, sim.t, interv=self, sim=sim):
+                    unvacc_inds = sc.findinds(~sim.people.vaccinated)
+                    vacc_probs[unvacc_inds] = self.prob  # Assign equal vaccination probability to everyone
+            vacc_inds = cvu.true(cvu.binomial_arr(vacc_probs))  # Calculate who actually gets vaccinated
+
+            if len(vacc_inds):
+                self.vaccinated[sim.t] = vacc_inds
+                sim.people.flows['new_vaccinations'] += len(vacc_inds)
+                sim.people.flows['new_vaccinated'] += len(vacc_inds)
+                if self.p.interval is not None:
+                    next_dose_day = sim.t + self.p.interval
+                    if next_dose_day < sim['n_days']:
+                        self.second_dose_days[next_dose_day] = vacc_inds
+
+            vacc_inds_dose2 = self.second_dose_days[sim.t]
+            if vacc_inds_dose2 is not None:
+                vacc_inds = np.concatenate((vacc_inds, vacc_inds_dose2), axis=None)
+                sim.people.flows['new_vaccinations'] += len(vacc_inds_dose2)
+
+            # Update vaccine attributes in sim
+            sim.people.vaccinated[vacc_inds] = True
+            sim.people.vaccine_source[vacc_inds] = self.index
+            self.vaccinations[vacc_inds] += 1
+            self.vaccination_dates[vacc_inds] = sim.t
+
+            # Update vaccine attributes in sim
+            sim.people.vaccinations[vacc_inds] = self.vaccinations[vacc_inds]
+            sim.people.date_vaccinated[vacc_inds] = self.vaccination_dates[vacc_inds]
+            cvi.init_nab(sim.people, vacc_inds, prior_inf=False)
+
+        return
+
