@@ -84,7 +84,7 @@ def handle_args(fig_args=None, plot_args=None, scatter_args=None, axis_args=None
     return args
 
 
-def handle_to_plot(which, to_plot, n_cols, sim, check_ready=True):
+def handle_to_plot(kind, to_plot, n_cols, sim, check_ready=True):
     ''' Handle which quantities to plot '''
 
     # Check that results are ready
@@ -94,20 +94,16 @@ def handle_to_plot(which, to_plot, n_cols, sim, check_ready=True):
 
     # If not specified or specified as a string, load defaults
     if to_plot is None or isinstance(to_plot, str):
-        if which == 'sim':
-            to_plot = cvd.get_sim_plots(to_plot)
-        elif which =='scens':
-            to_plot = cvd.get_scen_plots(to_plot)
-        else:
-            errormsg = f'"which" must be "sim" or "scens", not "{which}"'
-            raise NotImplementedError(errormsg)
+        to_plot = cvd.get_default_plots(to_plot, kind=kind, sim=sim)
 
     # If a list of keys has been supplied
     if isinstance(to_plot, list):
         to_plot_list = to_plot # Store separately
         to_plot = sc.odict() # Create the dict
+        reskeys = sim.result_keys()
         for reskey in to_plot_list:
-            to_plot[sim.results[reskey].name] = [reskey] # Use the result name as the key and the reskey as the value
+            name = sim.results[reskey].name if reskey in reskeys else sim.results['strain'][reskey].name
+            to_plot[name] = [reskey] # Use the result name as the key and the reskey as the value
 
     to_plot = sc.odict(sc.dcp(to_plot)) # In case it's supplied as a dict
 
@@ -373,7 +369,7 @@ def set_line_options(input_args, reskey, resnum, default):
 
 #%% Core plotting functions
 
-def plot_sim(sim, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot_args=None,
+def plot_sim(to_plot=None, sim=None, do_save=None, fig_path=None, fig_args=None, plot_args=None,
          scatter_args=None, axis_args=None, fill_args=None, legend_args=None, date_args=None,
          show_args=None, mpl_args=None, n_cols=None, grid=False, commaticks=True,
          setylim=True, log_scale=False, colors=None, labels=None, do_show=None, sep_figs=False,
@@ -387,18 +383,30 @@ def plot_sim(sim, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot
     fig, figs = create_figs(args, sep_figs, fig, ax)
 
     # Do the plotting
+    strain_keys = sim.result_keys('strain')
     for pnum,title,keylabels in to_plot.enumitems():
         ax = create_subplots(figs, fig, ax, n_rows, n_cols, pnum, args.fig, sep_figs, log_scale, title)
         for resnum,reskey in enumerate(keylabels):
-            res = sim.results[reskey]
             res_t = sim.results['t']
-            color = set_line_options(colors, reskey, resnum, res.color) # Choose the color
-            label = set_line_options(labels, reskey, resnum, res.name) # Choose the label
-            if res.low is not None and res.high is not None:
-                ax.fill_between(res_t, res.low, res.high, color=color, **args.fill) # Create the uncertainty bound
-            ax.plot(res_t, res.values, label=label, **args.plot, c=color) # Actually plot the sim!
+            if reskey in strain_keys:
+                res = sim.results['strain'][reskey]
+                ns = sim['n_strains']
+                strain_colors = sc.gridcolors(ns)
+                for strain in range(ns):
+                    color = strain_colors[strain]  # Choose the color
+                    label = 'wild type' if strain == 0 else sim['strains'][strain-1].label
+                    if res.low is not None and res.high is not None:
+                        ax.fill_between(res_t, res.low[strain,:], res.high[strain,:], color=color, **args.fill)  # Create the uncertainty bound
+                    ax.plot(res_t, res.values[strain,:], label=label, **args.plot, c=color)  # Actually plot the sim!
+            else:
+                res = sim.results[reskey]
+                color = set_line_options(colors, reskey, resnum, res.color)  # Choose the color
+                label = set_line_options(labels, reskey, resnum, res.name)  # Choose the label
+                if res.low is not None and res.high is not None:
+                    ax.fill_between(res_t, res.low, res.high, color=color, **args.fill)  # Create the uncertainty bound
+                ax.plot(res_t, res.values, label=label, **args.plot, c=color)  # Actually plot the sim!
             if args.show['data']:
-                plot_data(sim, ax, reskey, args.scatter, color=color) # Plot the data
+                plot_data(sim, ax, reskey, args.scatter, color=color)  # Plot the data
             if args.show['ticks']:
                 reset_ticks(ax, sim, args.date) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
         if args.show['interventions']:
@@ -409,7 +417,7 @@ def plot_sim(sim, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot
     return tidy_up(fig, figs, sep_figs, do_save, fig_path, do_show, args)
 
 
-def plot_scens(scens, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot_args=None,
+def plot_scens(to_plot=None, scens=None, do_save=None, fig_path=None, fig_args=None, plot_args=None,
          scatter_args=None, axis_args=None, fill_args=None, legend_args=None, date_args=None,
          show_args=None, mpl_args=None, n_cols=None, grid=False, commaticks=True, setylim=True,
          log_scale=False, colors=None, labels=None, do_show=None, sep_figs=False, fig=None, ax=None, **kwargs):
@@ -430,13 +438,27 @@ def plot_scens(scens, to_plot=None, do_save=None, fig_path=None, fig_args=None,
             resdata = scens.results[reskey]
             for snum,scenkey,scendata in resdata.enumitems():
                 sim = scens.sims[scenkey][0] # Pull out the first sim in the list for this scenario
-                res_y = scendata.best
-                color = set_line_options(colors, scenkey, snum, default_colors[snum]) # Choose the color
-                label = set_line_options(labels, scenkey, snum, scendata.name) # Choose the label
-                ax.fill_between(scens.tvec, scendata.low, scendata.high, color=color, **args.fill) # Create the uncertainty bound
-                ax.plot(scens.tvec, res_y, label=label, c=color, **args.plot) # Plot the actual line
-                if args.show['data']:
-                    plot_data(sim, ax, reskey, args.scatter, color=color) # Plot the data
+                strain_keys = sim.result_keys('strain')
+                if reskey in strain_keys:
+                    ns = sim['n_strains']
+                    strain_colors = sc.gridcolors(ns)
+                    for strain in range(ns):
+                        res_y = scendata.best[strain,:]
+                        color = strain_colors[strain]  # Choose the color
+                        label = 'wild type' if strain == 0 else sim['strains'][strain - 1].label
+                        ax.fill_between(scens.tvec, scendata.low[strain,:], scendata.high[strain,:], color=color, **args.fill)  # Create the uncertainty bound
+                        ax.plot(scens.tvec, res_y, label=label, c=color, **args.plot)  # Plot the actual line
+                        if args.show['data']:
+                            plot_data(sim, ax, reskey, args.scatter, color=color)  # Plot the data
+                else:
+                    res_y = scendata.best
+                    color = set_line_options(colors, scenkey, snum, default_colors[snum])  # Choose the color
+                    label = set_line_options(labels, scenkey, snum, scendata.name)  # Choose the label
+                    ax.fill_between(scens.tvec, scendata.low, scendata.high, color=color, **args.fill)  # Create the uncertainty bound
+                    ax.plot(scens.tvec, res_y, label=label, c=color, **args.plot)  # Plot the actual line
+                    if args.show['data']:
+                        plot_data(sim, ax, reskey, args.scatter, color=color)  # Plot the data
+
                 if args.show['interventions']:
                     plot_interventions(sim, ax) # Plot the interventions
                 if args.show['ticks']:
@@ -447,7 +469,7 @@ def plot_scens(scens, to_plot=None, do_save=None, fig_path=None, fig_args=None,
     return tidy_up(fig, figs, sep_figs, do_save, fig_path, do_show, args)
 
 
-def plot_result(sim, key, fig_args=None, plot_args=None, axis_args=None, scatter_args=None,
+def plot_result(key, sim=None, fig_args=None, plot_args=None, axis_args=None, scatter_args=None,
                 date_args=None, mpl_args=None, grid=False, commaticks=True, setylim=True, color=None, label=None,
                 do_show=None, do_save=False, fig_path=None, fig=None, ax=None, **kwargs):
     ''' Plot a single result -- see Sim.plot_result() for documentation '''
@@ -728,7 +750,7 @@ def plotly_sim(sim, do_show=False):
 
     go = import_plotly() # Load Plotly
     plots = []
-    to_plot = cvd.get_sim_plots()
+    to_plot = cvd.get_default_plots()
     for p,title,keylabels in to_plot.enumitems():
         fig = go.Figure()
         for key in keylabels:
